/*
 *  complex.h
 *
 *  Author: M. Reza Mozaffari
 *  Physics Group, University of Qom
 *  Created on: Jan. 2021
 */

#ifndef COMPLEX_H_
#define COMPLEX_H_

#include <cmath>
#include <cstdio>

class complex
{
private:
    double x, y;

public:
    complex();
    complex(double, double);

    void print();
    double get_real();
    double get_imag();
    void set_real(double);
    void set_imag(double);

    complex operator=(complex);
    complex operator+=(complex);
    complex operator-=(complex);
    complex operator*=(complex);
    complex operator*=(double);
    complex operator/=(double);
    complex operator/=(complex);

    complex conj();
    double norm();
    double arg();
    complex power(int);
};

void print(complex);
complex conj(complex);
double norm(complex);
double arg(complex);
complex power(complex, int);

complex operator+(complex, complex);
complex operator-(complex, complex);
complex operator*(double, complex);
complex operator*(complex, complex);
complex operator*(complex, double);
complex operator/(complex, complex);
complex operator/(complex, double);
complex operator/(double, complex);

#endif // COMPLEX_H_